#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#


#
# Purpose:
#   - This class contains methods pertaining to sending and receiving data to the
#     Pushpins through the IR to serial dongle.  
#
# Assumptions:
#   - Uses the pyserial serial module to control the serial port.
#
# History:
#   - 16 Mar 2004 by Josh Lifton.  Adapted from 17 Oct 2003 version of
#     Tribble debugger.
#   - 12 Jul 2004 by Michael Broxton.  Seperated this class from the text debugger.
#     Added support for multiple serial ports.  
#

import serial
import string, sys, time, timeit, os
from binascii import a2b_hex
from binascii import b2a_hex
from DataFidelity import crc8
from PushpinOS import PushpinOS
from PushpinPacket import PushpinPacket
from Globals import *
from NotificationCenter import *

## allNodes = map(string.upper, map(b2a_hex, map(chr, range(1,0x42))))
## allNodes.remove('2E')
## allNodes.remove('3B')
## allNodes.remove('21')
## allNodes.remove('3D')
## allNodes.remove('32')
## allNodes.remove('23')
## allNodes.remove('24')
## allNodes.remove('15')
## allNodes.remove('1B')
## allNodes.remove('41')

class DataLink(NotificationCenter):
    def __init__ (self, controller, serialPort = DEFAULT_COMM_PORT):
        self.serialPort = serialPort
        self.controller = controller
        self.s = None
        #
        # Initialize the Notification Center Machinery
        #
        NotificationCenter.__init__(self)

#----------------------- Managing serial Ports ------------------------------

    def connectSerialPort(self, guiobject):
        if (not self.issOK()) or (not self.isOpen()):
            if ( not self.openSerialPort() ):
                isConnected = False
            else:
                isConnected = True
        else:
            isConnected = True

        if guiobject == SPOTLIGHT:
            self.controller.SetSpotlightButtonStatus(isConnected)
        elif guiobject == DONGLE:
            self.controller.SetDongleButtonStatus(isConnected)
        return isConnected
                
    def disconnectSerialPort(self, guiobject):
        if self.issOK() and self.isOpen():
            if not self.closeSerialPort():
                isConnected = True
            else:
                isConnected = False
        else:
            isConnected = False

        if guiobject == SPOTLIGHT:
            self.controller.SetSpotlightButtonStatus(isConnected)
        elif guiobject == DONGLE:
            self.controller.SetDongleButtonStatus(isConnected)
        return not isConnected

    def openSerialPort(self) :
        print 'Opening serial port ' + str(self.serialPort) + '... ' 
        try: self.s = serial.Serial(port = self.serialPort,
                                    baudrate = DEFAULT_BAUD_RATE,
                                    timeout = DEFAULT_TIMEOUT)
        except serial.SerialException, detail:
            print >> self.controller.pd_stderr , "Failed!" , str(detail) , "\n"
            return False
        else:
            print "Success!\nSerial port opened on " + self.s.portstr + "\n"
            return True


    def closeSerialPort(self) :
        print 'Closing serial port ' + str(self.serialPort) + ' (' + self.s.portstr + ')... ' 
        try: self.s.close()
        except serial.SerialException, detail:
            print >> self.controller.pd_stderr , "Failed!  " , str(detail) , "\n"
            return False
        else:
            print "Success! \n\n"
            return True

    def clearSerialBuffer(self) :
        self.s.flush()

    def setSerialPort(self, port) :
        if self.s == None:
            self.serialPort = port
        else:
            self.closeSerialPort()
            self.serialPort = port
            self.openSerialPort()

    def isOpen(self):
        return self.s.isOpen()

    def getPortStr(self):
        return self.s.portstr

    def issOK(self):
        return not (self.s == None)

#----------------------- Basic Packet Sending Primitives --------------------

    def sendBasicPacket(self,
                        toAddress=GLOBALADDRESS,
                        fromAddress=IDEADDRESS,
                        packetType=BROADCAST,
                        sequenceNumber=0,
                        content=[]) :
        packetToSend = [toAddress, fromAddress, packetType]
        packetToSend.append(str('%02X' % sequenceNumber))
        contentSize = str('%04X' % len(content))
        packetToSend.append(contentSize[-2:])
        packetToSend.append(contentSize[0:2])
        packetToSend = a2b_hex(string.join(packetToSend, sep=''))
        packetToSend = packetToSend + chr(crc8(packetToSend))
        content = a2b_hex(string.join(content, sep=''))
        packetToSend = packetToSend + chr(crc8(content))
        packetToSend = packetToSend + content

        self.s.write(packetToSend)

    def getBasicPacket(self) :
        ppPacket = PushpinPacket([])
        character = self.s.read()
        while character :
            ppPacket.addBytes(character)
            if ppPacket.isComplete :
                break
            character = self.s.read()
        return ppPacket


#----------------- Specific Packet Types ---------------------------------

    def sendSilencePacket(self) :
        self.sendBasicPacket(content=[BEGIN_DATA_HARVEST])

    def sendWaitPacket(self) :
        self.sendBasicPacket(content=[WAIT_FOR_OS])

    def sendStartPacket(self) :
        self.sendBasicPacket(content=[DONT_WAIT_FOR_OS])

    def sendQueryPacket(self, NeighborhoodID) :
        if (NeighborhoodID > 255) :
            return False

        idStr = str('%02X' % NeighborhoodID)

        self.sendBasicPacket(content=[ROUTED_REQUEST_MESSAGE, # type of message
                                      IDEADDRESS,             # requester id
                                      idStr[0:2],             # requested id
                                      '00',                   # hop count
                                      IDEADDRESS])            # return path
        self.s.setTimeout(DATA_COLLECTION_TIMEOUT)
        p = self.getBasicPacket()
        while p.packetList :
            if p.isCorrect :
                if p.contentList :
                    if ((p.contentList[0] == ROUTED_REPLY_MESSAGE) and  # message type
                        (p.contentList[1] == IDEADDRESS) and            # requesting node id
                        (p.contentList[2] == idStr)) :                  # responding node id
                        ##p.printHeader()
                        ##print p.contentList 
                        ##print ('x: ' +
                        ##       str(self.getXCoordinate(p.contentList)) +
                        ##       '\ty: ' +
                        ##       str(self.getYCoordinate(p.contentList)))
                        ##print 'Data retrieved for node ' + idStr
                        #
                        # Notify all listeners that a new packet is available for processing.
                        #
                        self.NotifyListeners(p, p.contentList[0])
                        break;
            else :
                print >> self.controller.pd_stderr , 'Bad packet'
            p = self.getBasicPacket()
        self.s.setTimeout(DEFAULT_TIMEOUT)

    
    def sendOSPacket(self, pushpinOS) :
        delay = self.computeDelayBetweenPackets();
        if pushpinOS == None :
            loadPushpinOS()
        pushpinOS.printAttributes()
        osVersionLowByte = pushpinOS.version[2:4]
        osVersionHighByte = pushpinOS.version[0:2]
        osSizeHexString = str('%04X' % len(pushpinOS.osByteString))
        osHeader=[OS_HEADER_MESSAGE,
                  osSizeHexString[2:4],
                  osSizeHexString[0:2],
                  osVersionLowByte,
                  osVersionHighByte,
                  str('%02X' % pushpinOS.CRC),
                  str('%02X' % pushpinOS.stack)]
        self.sendBasicPacket(content=osHeader)
        # Takes 10ms to erase a page of flash.  Maximum 64 pages to erase.
        time.sleep(2)
        # meme:  print 'Number of \'pass\' intstructions to delay between bytes: ' + str(delay)
        print 'Sent OS header: ' 
        print osHeader 
        t0 = time.time()
        for b in pushpinOS.osByteString :
            self.s.flush()
            self.s.write(b)
            if (os.name == 'nt') :  # Fix for windoze
                for i in range(delay) :
                    pass
            else :
                delayEpoch = time.time();
                while (time.time() - delayEpoch < TIME_DELAY_FOR_OSSEND) :
                    pass
        t1 = time.time()
        print 'Sent OS in ' + str(t1 - t0) + ' seconds.' 

    def sendIDERequestPacket(self) :
        self.sendBasicPacket(content=[IDE_REQUEST_MESSAGE])
        returnedTestPacket = self.getBasicPacket()
        if returnedTestPacket.isCorrect :
            tofDump = returnedTestPacket.contentList
            if len(tofDump) == IDE_DUMP_MESSAGE_SIZE :
                print '                         type: ' + tofDump[0]
                print '                           id: ' + tofDump[1]
                print '           ping1LocalDistance: ' + tofDump[3] + tofDump[2]
                print '    ping1MinimumKnownDistance: ' + tofDump[5] + tofDump[4]
                print '  ping1MinimumKnownDistanceID: ' + tofDump[6]
                print '                ping1AnchorID: ' + tofDump[7]
                print '  ping1AnchorDistanceToPinger: ' + tofDump[9] + tofDump[8]
                print '       ping1AnchorXCoordinate: ' + tofDump[13] + tofDump[12] + tofDump[11] + tofDump[10]
                print '       ping1AnchorYCoordinate: ' + tofDump[17] + tofDump[16] + tofDump[15] + tofDump[14]
                print ' ping1MinimumHopCountToAnchor: ' + tofDump[18]
                print '           ping2LocalDistance: ' + tofDump[20] + tofDump[19]
                print '    ping2MinimumKnownDistance: ' + tofDump[22] + tofDump[21]
                print '  ping2MinimumKnownDistanceID: ' + tofDump[23]
                print '                ping2AnchorID: ' + tofDump[24]
                print '  ping2AnchorDistanceToPinger: ' + tofDump[26] + tofDump[25]
                print '       ping2AnchorXCoordinate: ' + tofDump[30] + tofDump[29] + tofDump[28] + tofDump[27]
                print '       ping2AnchorYCoordinate: ' + tofDump[34] + tofDump[33] + tofDump[32] + tofDump[31]
                print ' ping2MinimumHopCountToAnchor: ' + tofDump[35]
                print '           ping3LocalDistance: ' + tofDump[37] + tofDump[36]
                print '    ping3MinimumKnownDistance: ' + tofDump[39] + tofDump[38]
                print '  ping3MinimumKnownDistanceID: ' + tofDump[40]
                print '                ping3AnchorID: ' + tofDump[41]
                print '  ping3AnchorDistanceToPinger: ' + tofDump[43] + tofDump[42]
                print '       ping3AnchorXCoordinate: ' + tofDump[47] + tofDump[46] + tofDump[45] + tofDump[44]
                print '       ping3AnchorYCoordinate: ' + tofDump[51] + tofDump[50] + tofDump[49] + tofDump[48]
                print ' ping3MinimumHopCountToAnchor: ' + tofDump[52]
                print '                 X-coordinate: ' + tofDump[56] + tofDump[55] + tofDump[54] + tofDump[53]
                print '                 Y-coordinate: ' + tofDump[60] + tofDump[59] + tofDump[58] + tofDump[57]
                print '            numberOfNeighbors: ' + tofDump[61]
            else :
                print >> self.controller.pd_stderr , 'CONTENT IS TOO SHORT FOR AN IDEDUMPMESSAGE'
        

# --------------------------- Utility Methods -------------------------

    # A hack to fix poor windoze timing granularity...
    def computeDelayBetweenPackets(self) :
        t = timeit.Timer("pass");
        # timeit() computes the time necessary for 1e6 executions of pass
        timeForOnePass = t.timeit(1000000) / 1.0e6; 
        numberOfPasses = int(WINDOZE_TIME_DELAY_FOR_OSSEND / timeForOnePass)   
        return numberOfPasses

    def getXCoordinate(self, replyContent) :
        if replyContent[0] == ROUTED_REPLY_MESSAGE :
            return hexToSignedInt(replyContent[5] + replyContent[4])
        else :
            return None

    def getYCoordinate(self, replyContent) :
        if replyContent[0] == ROUTED_REPLY_MESSAGE :
            return hexToSignedInt(replyContent[7] + replyContent[6])
        else :
            return None


    def monitorNetwork() :
        while True :
            p = self.getBasicPacket()
            if p.packetList :
                p.printHeader()
                print p.contentList


    def retrieveAllData() :
        while True :
            outputFileName = raw_input('File name to write to (should end in .dat): ')
            if ((len(outputFileName) > 4) and (outputFileName[-4:] == '.dat')) :
                break;
        dataArray = range(len(allNodes))
        for index in range(len(allNodes)) :
            nodeID = allNodes[index]
            dataArray[index] = nodeID
            self.sendBasicPacket(content=[ROUTED_REQUEST_MESSAGE, # type of message
                                     IDEADDRESS,             # requester id
                                     nodeID,                 # requested id
                                     '00',                   # hop count
                                     IDEADDRESS])            # return path
            self.s.setTimeout(DATA_COLLECTION_TIMEOUT)
            p = self.getBasicPacket()
            while p.packetList :
                if p.isCorrect :
                    if p.contentList :
                        if ((p.contentList[0] == ROUTED_REPLY_MESSAGE) and
                            (p.contentList[1] == IDEADDRESS) and
                            (p.contentList[2] == nodeID)) :
                            dataArray[index] = p.contentList
                            #      p.printHeader()
                            self.controller.p.contentList
                            print 'x: ' + str(getXCoordinate(p.contentList)) + '\ty: ' + str(getYCoordinate(p.contentList))
                            print 'Data retrieved for node ' + nodeID
                            break;
                else :
                    print >> self.controller.pd_stderr , 'Bad packet'
                p = self.getBasicPacket()
            time.sleep(0.5)
            clearSerialBuffer()
        self.s.setTimeout(DEFAULT_TIMEOUT)
        missingNodes = []
        dataFile = open(outputFileName, 'w')
        for dataLog in dataArray :
            if len(dataLog) == 2 :
                missingNodes.append(dataLog)
            else :
                dataFile.write(dataLog[2] + '\t' +              # node id
                               dataLog[3] + '\t' +              # hop count
                               str(hexToSignedInt(dataLog[5] + dataLog[4])) + '\t' + # x-coordinate
                               str(hexToSignedInt(dataLog[7] + dataLog[6])) + '\n')  # y-coordinate
        dataFile.close()
        print 'Data written to ' + dataFile.name
        if missingNodes :
            print 'No data for nodes ' + string.join(missingNodes, ', ')
        else :
            print 'Data for all listed nodes successfully collected.'
