#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#----------------------------------------------------------------------------
# Name:         Interface.py
#
# Purpose:      The graphical front end for the Pushpin Debugger.  This file
#               contains routines that define the basic GUI layout of the Pushpin
#               debugger.  This includes menus, the main window, the top button bar,
#               the status bar, and the log window (which is the bottom half of the
#               main window.  The only GUI elements not defined in this file are the
#               function-specific notebook panels.  These are each defined in a seperate
#               python source file.
#
# Author:       Michael Broxton and Josh Lifton
#
# Created:      A long time ago, in a galaxy far, far away...
# Copyright:    (c) 2004 by MIT Media Laboratory
#----------------------------------------------------------------------------


#
# Import all of the wxPython GUI package, the PushpinDebugger Globals, and
# the sys package.
#
from wxPython.wx import *
from Globals import *
import sys
from time import localtime


#
# ID Definitions for the event handling system.
#
ID_FILE_EXIT = wxNewId() 
ID_FILE_ABOUT = wxNewId()
ID_BUILD_WAIT = wxNewId()
ID_BUILD_START = wxNewId()
ID_BUILD_COMPILE = wxNewId()
ID_BUILD_SEND = wxNewId()
ID_BUILD_INFO = wxNewId()
ID_DEBUG_SILENCE = wxNewId()
ID_DEBUG_QUERY = wxNewId()
ID_DEBUG_DUMP = wxNewId()
ID_OPTIONS_SPOTLIGHT_PORT_MENU = wxNewId()
ID_OPTIONS_SPOTLIGHT_PORT1 = wxNewId()
ID_OPTIONS_SPOTLIGHT_PORT2 = wxNewId()
ID_OPTIONS_IRSERIALDONGLE_PORT_MENU = wxNewId()
ID_OPTIONS_IRSERIALDONGLE_PORT1 = wxNewId()
ID_OPTIONS_IRSERIALDONGLE_PORT2 = wxNewId()
ID_PLOT_ENABLEZOOM = wxNewId()
ID_PLOT_AUTOZOOM = wxNewId()
ID_PLOT_PAGESETUP = wxNewId()
ID_PLOT_PRINTPREVIEW = wxNewId()
ID_PLOT_PRINT = wxNewId()
ID_PLOT_SAVE = wxNewId()

# Toolbar button ID's
ID_OS_COMPILE = wxNewId()
ID_OS_SEND = wxNewId()
ID_SPOTLIGHT_CONNECT = wxNewId()
ID_SPOTLIGHT_DISCONNECT = wxNewId()
ID_SPOTLIGHT_WAIT = wxNewId()
ID_SPOTLIGHT_START = wxNewId()
ID_SPOTLIGHT_SILENCE = wxNewId()
ID_DONGLE_CONNECT = wxNewId()
ID_DONGLE_DISCONNECT = wxNewId()
ID_DONGLE_ALL_DATA = wxNewId()
ID_DONGLE_QUERY = wxNewId()
ID_DONGLE_MONITOR = wxNewId()
ID_QUERY_TEXTCTRL = wxNewId()

ID_TOOLBAR = wxNewId()
ID_SAVE_LIST = 1000




## Create a new frame class, derived from the wxPython Frame.  This is where
## the main parts of the GUI are set up -- Specifically, the menus, toolbar
## and status window. 
##
class MainWindow(wxFrame):
    
    def __init__(self, controller, id, title,  style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE):
        #
        # First, make a local pointer to the controller class so that we can refer to it later
        # when be begin connection actions.
        #
        self.controller = controller
        self.notebookPanels = []
        
        #
        # Call the base class' __init__ method to create the frame
        #
        wxFrame.__init__(self, NULL, id, title, size = (1024, 768), style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE|wxMAXIMIZE)
        
        #
        # This line is critical for reading in image data from files. (And it took
        # me an entire afternoon one day to find it in the documentation...)  Don't
        # let this happen to you, kids.
        #
        wxInitAllImageHandlers()

        #
        # Initialize various components of the GUI
        #
        self.SetupMenubar()
        self.SetupToolbar()
        self.SetupStatusbar()
        
        #
        # Now that the basic window is defined, we set up the split window.
        # The bottom half of the main window contains the
        # text based command feedback (log) area.  The top half contains all other content.
        #
        self.splitter = ProportionalSplitter(self, proportion = 0.66)

        #
        # Create notebook ( tabbed windows ) for the upper half of the splitter.
        #
        self.noteBook = wxNotebook(self.splitter, -1, wxPoint(0,0), wxSize(0,0), wxNB_FIXEDWIDTH | wxNB_BOTTOM)
        self.Bind(EVT_NOTEBOOK_PAGE_CHANGED,self.HandleNotebookAction)

        #
        # Now that we have set up upper splitter contents, we can set
        # up the log panel on the bottom window.  We will use this log area
        # for printing to both standard out and standard error.
        #
        self.logPanel = LogPanel(self.splitter)
        self.stdout = stdoutObj(self.logPanel.textControl)
        self.stderr = stderrObj(self.logPanel.textControl) 

        #
        # Finally, activate the splitter.
        #
        self.splitter.SplitHorizontally(self.noteBook, self.logPanel)

        #
        # Use some sizers to help keep everything in the window nicely proportioned
        # if the window is resized.
        #
        self.sizer = wxBoxSizer(wxVERTICAL)
        self.sizer.Add(self.splitter, 1, wxEXPAND)
        self.SetSizer(self.sizer)
        self.SetAutoLayout(1)
        
        #
        # Once everything has been set up, show the frame.
        #
        self.Show(true)


    #
    # Load the notebook tabs.  Each tab is a relatively self-contained application for interacting
    # with the Pushpins.  These tabs must be loaded after both the GUI and the data model have been
    # created by the main pushpin controller class, PushpinDebugger.
    #
    def loadNotebookPanels(self):
        panelPackage = __import__("Panels", globals())
        #
        # The active panels are listed in the Panels.__all__.
        # Edit Panels/__init__.py to change the list of currently
        # loadable panels.
        #
        for x in panelPackage.__all__:
            try:
                panelModule = __import__("Panels." + x, globals(), locals(), x)
                panel = panelModule.runPanel(self.noteBook, self.controller, self.MenuBar)
                if panel:
                    self.notebookPanels.append(panel)
                    self.noteBook.AddPage(panel, panel.Name())
                else:
                    print >> self.controller.pd_stderr, ("Error:  Could not load panel ", "Panels.", x)
            except:
                print >> self.controller.pd_stderr, "Error:  Exception occured when loading panel ", "Panels.", x
                print >> self.controller.pd_stderr, "\t", sys.exc_info()[0], "\n\t", sys.exc_info()[1].args[0]
            
    #---------------------------------------------------------------------
    #
    # Set up a basic menu bar.  
    def SetupMenubar(self):
        menubar = wxMenuBar()
        self.MenuBar = menubar
        
        menu = wxMenu()
        menu.Append(ID_FILE_ABOUT, "About", "About This Program")
        menu.Append(ID_FILE_EXIT, "Quit\tCTRL-Q", "Exit the Program")
        menubar.Append(menu, "File")

        menu = wxMenu()
        menu.Append(ID_BUILD_WAIT, "Wait\tCTRL-W", "Tell Pushpin Bootloader to Wait for OS")
        menu.Append(ID_BUILD_START, "Start\tCTRL-S", "Tell Pushpin Bootloader to Start")
        menu.Append(ID_BUILD_COMPILE, "Compile OS\tCTRL-C", "Compile a new OS")
        menu.Append(ID_BUILD_SEND, "Send OS\tCTRL-O", "Send a New OS")
        menu.Append(ID_BUILD_INFO, "OS Info\tCTRL-I", "Print Info on the Current OS Build")
        menubar.Append(menu, "Build")

        menu = wxMenu()
        menu.Append(ID_DEBUG_SILENCE, "Silence", "Stop the current Application Code")
        menu.Append(ID_DEBUG_QUERY, "Query\tCTRL-U", "Query a Specific Node")
        menu.Append(ID_DEBUG_DUMP, "Dump\tCTRL-D", "Dump All Information From a Node.")
        menubar.Append(menu, "Debug")

        menu = wxMenu()
        spotlightSubMenu = wxMenu()
        spotlightSubMenu.Append(ID_OPTIONS_SPOTLIGHT_PORT1, "Port 1", "", wxITEM_RADIO)
        spotlightSubMenu.Append(ID_OPTIONS_SPOTLIGHT_PORT2, "Port 2", "", wxITEM_RADIO)
        if (DEFAULT_SPOTLIGHT_SERIALPORT == 0):
            spotlightSubMenu.Check(ID_OPTIONS_SPOTLIGHT_PORT1, True)
        else:
            spotlightSubMenu.Check(ID_OPTIONS_SPOTLIGHT_PORT2, True)
        menu.AppendMenu(ID_OPTIONS_SPOTLIGHT_PORT_MENU, "Spotlight Programmer Port", spotlightSubMenu, "")
        dongleSubMenu = wxMenu()
        dongleSubMenu.Append(ID_OPTIONS_IRSERIALDONGLE_PORT1, "Port 1", "", wxITEM_RADIO)
        dongleSubMenu.Append(ID_OPTIONS_IRSERIALDONGLE_PORT2, "Port 2", "", wxITEM_RADIO)
        if (DEFAULT_DONGLE_SERIALPORT == 0):
            dongleSubMenu.Check(ID_OPTIONS_IRSERIALDONGLE_PORT1, True)
        else:
            dongleSubMenu.Check(ID_OPTIONS_IRSERIALDONGLE_PORT2, True)
        menu.AppendMenu(ID_OPTIONS_IRSERIALDONGLE_PORT_MENU, "IR-Serial Dongle Port", dongleSubMenu)
        menubar.Append(menu, "&Options")

##         menu = wxMenu()
##         menu.Append(ID_PLOT_ENABLEZOOM, 'Enable Zoom', kind=wxITEM_CHECK)
##         menu.Append(ID_PLOT_AUTOZOOM, 'Zoom Auto')
##         menu.Append(ID_PLOT_PAGESETUP, 'Page Setup...')
##         menu.Append(ID_PLOT_PRINTPREVIEW, 'Print Preview...')
##         menu.Append(ID_PLOT_PRINT, 'Print...')
##         menu.Append(ID_PLOT_SAVE, 'Save Plot...')
##         menubar.Append(menu, '&Plot')

##         menu = wxMenu()
##         menu.Append(ID_PRINT_NODES,'Print Nodes')
##         menubar.Append(menu, 'Node')

        self.SetMenuBar(menubar)
##        menubar.EnableTop(4,FALSE)      


        #
        # Create event bindings for each of the menu options specified above.
        # 
        EVT_MENU(self.controller, ID_FILE_ABOUT, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_FILE_EXIT, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_BUILD_WAIT, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_BUILD_START, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_BUILD_COMPILE, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_BUILD_SEND, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_BUILD_INFO, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_DEBUG_SILENCE, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_DEBUG_QUERY, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_DEBUG_DUMP, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_OPTIONS_SPOTLIGHT_PORT1, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_OPTIONS_SPOTLIGHT_PORT2, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_OPTIONS_IRSERIALDONGLE_PORT1, self.controller.HandleMenuAction)
        EVT_MENU(self.controller, ID_OPTIONS_IRSERIALDONGLE_PORT2, self.controller.HandleMenuAction)
##         EVT_MENU(self.controller, ID_PLOT_ENABLEZOOM, self.controller.HandleMenuAction)
##         EVT_MENU(self.controller, ID_PLOT_AUTOZOOM, self.controller.HandleMenuAction)
##         EVT_MENU(self.controller, ID_PLOT_PAGESETUP, self.controller.HandleMenuAction)
##         EVT_MENU(self.controller, ID_PLOT_PRINTPREVIEW, self.controller.HandleMenuAction)
##         EVT_MENU(self.controller, ID_PLOT_PRINT, self.controller.HandleMenuAction)
##         EVT_MENU(self.controller, ID_PLOT_SAVE, self.controller.HandleMenuAction)



    def HandleNotebookAction(self, event):
        pass

# Meme: Add more general interface for notebook tabs here.
#        if event.GetSelection()==1:
#            
#            self.MenuBar.EnableTop(4,TRUE)
#        else:
#            self.MenuBar.EnableTop(4,FALSE)


    # ----------------------------------------------------------------------------------------
    #
    # Build the Toolbar
    #
    def SetupToolbar(self):
        tb = self.CreateToolBar(wxTB_HORIZONTAL|wxTB_TEXT)
        self.ToolBar = tb
        tb.SetToolBitmapSize((45, 45))  # this required for non-standard size buttons on MSW
        tb.SetToolSeparation(30)

        #
        #  The current state of the Spotlight and Dongle Connection is
        #  needed to properly setup Toolbar
        #
        self.isSpotlightConnected = False
        self.isDongleConnected = False
        
        bitmap = wxBitmap('resources/connect.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_SPOTLIGHT_CONNECT, bitmap, "", "Connect Spotlight")
        EVT_TOOL(self.controller, ID_SPOTLIGHT_CONNECT, self.controller.HandleToolbarAction)

        bitmap = wxBitmap('resources/os_compile.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_OS_COMPILE, bitmap, "", "Compile OS")
        EVT_TOOL(self.controller, ID_OS_COMPILE, self.controller.HandleToolbarAction)

        bitmap = wxBitmap('resources/os_send.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_OS_SEND, bitmap, "", "Send OS")
        EVT_TOOL(self.controller, ID_OS_SEND, self.controller.HandleToolbarAction)

        bitmap = wxBitmap('resources/wait.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_SPOTLIGHT_WAIT, bitmap, "", "Spotlight Wait")
        EVT_TOOL(self.controller, ID_SPOTLIGHT_WAIT, self.controller.HandleToolbarAction)

        bitmap = wxBitmap('resources/start.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_SPOTLIGHT_START, bitmap, "", "Start Stoplight")
        EVT_TOOL(self.controller, ID_SPOTLIGHT_START, self.controller.HandleToolbarAction)

        bitmap = wxBitmap('resources/silence.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_SPOTLIGHT_SILENCE, bitmap, "", "Silence Spotlight")
        EVT_TOOL(self.controller, ID_SPOTLIGHT_SILENCE, self.controller.HandleToolbarAction)

        tb.AddSeparator()

        bitmap = wxBitmap('resources/connect.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_DONGLE_CONNECT, bitmap, "", "Connect Dongle")
        EVT_TOOL(self.controller, ID_DONGLE_CONNECT, self.controller.HandleToolbarAction)

        bitmap = wxBitmap('resources/monitor.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_DONGLE_MONITOR, bitmap, "", "Monitor Dongle")
        EVT_TOOL(self.controller, ID_DONGLE_MONITOR, self.controller.HandleToolbarAction)

        bitmap = wxBitmap('resources/all_data.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_DONGLE_ALL_DATA, bitmap, "", "Dongle All Data")
        EVT_TOOL(self.controller, ID_DONGLE_ALL_DATA, self.controller.HandleToolbarAction)

        tb.AddSeparator()

        bitmap = wxBitmap('resources/query.gif', wxBITMAP_TYPE_GIF)
        tb.AddSimpleTool(ID_DONGLE_QUERY, bitmap, "", "Dongle Query")
        EVT_TOOL(self.controller, ID_DONGLE_QUERY, self.controller.HandleToolbarAction)

        self.queryTextCtrl = wxTextCtrl(tb, ID_QUERY_TEXTCTRL, "00", size=(30, -1), style = wxTE_PROCESS_ENTER)
        tb.AddControl(self.queryTextCtrl)
        
        tb.Realize()

    # --------------------------------------------------------------------
    #
    # Build the Status Bar
    #
    def SetupStatusbar(self):
        statusBar = self.CreateStatusBar()
        self.StatusBar = statusBar
        statusBar.SetFieldsCount( 3 )

        # Set the first field to variable length, second field to fixed.  
        statusBar.SetStatusWidths( [-1, 150, 150] )

        statusBar.SetStatusText( "Welcome to the Pushpin Debugger!!", 0 )
        statusBar.SetStatusText( "Spotlight: Disconnected",1)
        statusBar.SetStatusText( "Dongle: Disconnected",2)
         

    #---------------------------------------------------------------------
    #
    # The about box dialog.
    #
    def AboutBox(self):
        aboutString = ('Written by Michael Broxton and Josh Lifton.\n\n' +
                       'Special thanks to Danny Benitez, Uzoma Orji, and Yun Wu, \n' +
                       'for their tremendous help on this and other aspects of \n' +
                       'the Pushpin project.\n\n' +
                       '(c) 2004 MIT Media Laboratory')
        dlg = wxMessageDialog(self, aboutString, 'Pushpin Debugger', wxOK | wxICON_INFORMATION) 
        dlg.ShowModal() 
        dlg.Destroy()

    #---------------------------------------------------------------------
    #
    # TOOLBAR Changes
    # These two functions change the icon for the SpotLight and Dongle to indicate
    # whether or not they are connected to their assigned serial ports
    #
    
    def setSpotlightGUI(self, isConnected):
        if not (isConnected == self.isSpotlightConnected):
            if isConnected:
                self.isSpotlightConnected = True
                self.ToolBar.RemoveTool(ID_SPOTLIGHT_CONNECT)
                self.ToolBar.RemoveTool(ID_SPOTLIGHT_DISCONNECT)
                self.StatusBar.SetStatusText("Spotlight: Connected", 1)
                bitmap = wxBitmap('resources/disconnect.gif', wxBITMAP_TYPE_GIF)
                self.ToolBar.InsertSimpleTool(0,ID_SPOTLIGHT_DISCONNECT,bitmap, "", "Disconnect Spotlight")
                EVT_TOOL(self.controller, ID_SPOTLIGHT_DISCONNECT, self.controller.HandleToolbarAction)
                self.ToolBar.Realize()
            else:
                self.isSpotlightConnected = False
                self.ToolBar.RemoveTool(ID_SPOTLIGHT_CONNECT)
                self.ToolBar.RemoveTool(ID_SPOTLIGHT_DISCONNECT)
                self.StatusBar.SetStatusText( "Spotlight: Disconnected",1)
                bitmap = wxBitmap('resources/connect.gif', wxBITMAP_TYPE_GIF)
                self.ToolBar.InsertSimpleTool(0,ID_SPOTLIGHT_CONNECT,bitmap, "", "Connect Spotlight")
                EVT_TOOL(self.controller, ID_SPOTLIGHT_CONNECT, self.controller.HandleToolbarAction)
                self.ToolBar.Realize()
            

    def setDongleGUI(self,isConnected):
        if not (isConnected == self.isDongleConnected): 
            if isConnected:
                self.isDongleConnected = True
                self.ToolBar.RemoveTool(ID_DONGLE_CONNECT)
                self.ToolBar.RemoveTool(ID_DONGLE_DISCONNECT)
                self.StatusBar.SetStatusText( "Dongle: Connected", 2)
                bitmap = wxBitmap('resources/disconnect.gif', wxBITMAP_TYPE_GIF)
                self.ToolBar.InsertSimpleTool(7,ID_DONGLE_DISCONNECT, bitmap, "", "Disconnect Dongle")
                EVT_TOOL(self.controller, ID_DONGLE_DISCONNECT, self.controller.HandleToolbarAction)
                self.ToolBar.Realize()
            else:
                self.isDongleConnected = False
                self.ToolBar.RemoveTool(ID_DONGLE_CONNECT)
                self.ToolBar.RemoveTool(ID_DONGLE_DISCONNECT)
                self.StatusBar.SetStatusText( "Dongle: Disconnected",2)
                bitmap = wxBitmap('resources/connect.gif', wxBITMAP_TYPE_GIF)
                self.ToolBar.InsertSimpleTool(7, ID_DONGLE_CONNECT, bitmap, "", "Connect Dongle")
                EVT_TOOL(self.controller, ID_DONGLE_CONNECT, self.controller.HandleToolbarAction)
                self.ToolBar.Realize()
              
           
    #---------------------------------------------------------------------
    #
    # Status bar dialog    # MEME - THIS CODE DOES NOT YET WORK!!!!!
    #
    def createStatusDialog(self):
        self.statusDialog = wxDialog(self, -1, 'Command Progress', (200, 300), (400, 75))
        self.progressGauge =  wxGauge(self.statusDialog, -1, 100, (25, 25), (350, 25), wxGA_HORIZONTAL | wxGA_SMOOTH )
        self.progressGauge.SetBezelFace(3)
        self.progressGauge.SetShadowWidth(3)

        # Use some sizers to see layout options
        sizer=wxBoxSizer(wxHORIZONTAL)
        sizer.Add(self.progressGauge, 1, wxEXPAND)
        
        # Layout sizers
        self.statusDialog.SetSizer(sizer)
        self.statusDialog.SetAutoLayout(1)

        self.statusDialog.Show() 

#    def updateStatusDialog(self, percent):
#        self.progressGauge.SetValue( int(percent * 100) )

    def destroyStatusDialog(self):
        self.progressGauge.Destroy()
        self.progressGauge = None
        self.statusDialog.Destroy() 
        self.statusDialog = None


    #
    # This method is called by the System when the window is resized,
    # because of the association above.
    #
    def OnSize(self, event):
        size = event.GetSize()
        print "size:", size.width, size.height

    #
    # This method is called automatically when the CLOSE event is
    # sent to this window
    #
    def OnCloseWindow(self, event):
        # tell the window to kill itself
        self.OnFileExit()



#---------------------------------------------------------------------------
# UTILITY CLASSES (classes that are used by the main GUI class above)
#


#
# The proportional splitter class was taken from the wxPython cookbook.
# It splits the window according to a proportional value between 0 and 1.0.
#
class ProportionalSplitter(wxSplitterWindow):
    def __init__(self, parent, id = -1, proportion=0.66, size = wxDefaultSize):
        wxSplitterWindow.__init__(self,parent,id,wxPoint(0, 0),size,0)
        self.SetMinimumPaneSize(50) #the minimum size of a pane.
        self.proportion = proportion
        if (self.proportion < 0.0) or (self.proportion > 1.0):
            raise ValueError, "proportion value for ProportionalSplitter must be between 0 and 1."
        self.ResetSash()
        EVT_SIZE(self, self.OnReSize)
        EVT_SPLITTER_SASH_POS_CHANGED(self, id, self.OnSashChanged)
        # Hack to set sizes on first paint event
        EVT_PAINT(self, self.OnPaint)
        self.firstpaint = true

    def SplitHorizontally(self, win1, win2):
        if self.GetParent() is None: return false
        return wxSplitterWindow.SplitHorizontally(self,win1,win2,int(round(self.GetParent().GetSize().GetHeight() * self.proportion)))

    def SplitVertically(self, win1, win2):
        if self.GetParent() is None: return false
        return wxSplitterWindow.SplitVertically(self,win1,win2,int(round(self.GetParent().GetSize().GetWidth() * self.proportion)))

    def GetExpectedSashPosition(self):
        if self.GetSplitMode() == wxSPLIT_HORIZONTAL:
            tot = max(self.GetMinimumPaneSize(),self.GetParent().GetClientSize().height)
        else:
            tot = max(self.GetMinimumPaneSize(),self.GetParent().GetClientSize().width)
        return int(round(tot * self.proportion))

    def ResetSash(self):
        self.SetSashPosition(self.GetExpectedSashPosition())

    def OnReSize(self, event):
        "Window has been resized, so we need to adjust the sash based on self.proportion."
        self.ResetSash()
        event.Skip()

    def OnSashChanged(self, event):
        "We'll change self.proportion now based on where user dragged the sash."
        pos = float(self.GetSashPosition())
        if self.GetSplitMode() == wxSPLIT_HORIZONTAL:
            tot = max(self.GetMinimumPaneSize(),self.GetParent().GetClientSize().height)
        else:
            tot = max(self.GetMinimumPaneSize(),self.GetParent().GetClientSize().width)
        self.proportion = pos / tot
        event.Skip()

    def OnPaint(self,event):
        if self.firstpaint:
            if self.GetSashPosition() != self.GetExpectedSashPosition():
                self.ResetSash()
            self.firstpaint = false
        event.Skip()

#
# The log panel contains a scrolled window of text.
# 
class LogPanel(wxScrolledWindow): 
     def __init__(self, parent):
         self.parent = parent
         wxScrolledWindow.__init__(self, parent ,-1)
 
         self.textControl = wxTextCtrl(self, -1, style = wxTE_MULTILINE | wxTE_WORDWRAP | wxTE_READONLY )
         # Use some sizers to see layout options
         self.sizer=wxBoxSizer(wxVERTICAL)
         self.sizer.Add(self.textControl, 1, wxEXPAND)
         
         # Layout sizers
         self.SetSizer(self.sizer)
         self.SetAutoLayout(1)
          
         self.unit=20 
         width,height=self.textControl.GetSizeTuple() 
 
         self.SetScrollbars(self.unit, self.unit, width/self.unit, height/self.unit)         

     def AppendText(self, string):
         self.textControl.WriteText(string)

     def write(self, string):
         self.textControl.WriteText(string)

#
# This class is used to route stdout to a text area in the GUI.  By
# setting sys.stdout equal to an instance of this class, the print
# statement can be used to write to the text area specified by
# textArea, which must implement the WriteText() method.  Typically,
# the wxTextCtrl object is used.
#
class stdoutObj:
    def __init__(self, textArea):
        self.textArea = textArea

    def write(self, string):
        begin = self.textArea.GetInsertionPoint()
        self.textArea.WriteText(string)
        end = self.textArea.GetInsertionPoint()
        style = wxTextAttr()
        self.textArea.GetStyle(begin, style)
        style.SetTextColour(wxBLACK)
        self.textArea.SetStyle(begin, end, style)
        
#
# Similar to the above class, except it attempts to color the text of an error message wxRED.
#
class stderrObj:
    def __init__(self, textArea):
        self.textArea = textArea
        
    def write(self, string):
        begin = self.textArea.GetInsertionPoint()
        self.textArea.WriteText(string)
        end = self.textArea.GetInsertionPoint()
        style = wxTextAttr()
        self.textArea.GetStyle(begin, style)
        style.SetTextColour(wxRED)
        self.textArea.SetStyle(begin, end, style)

#---------------------------------------------------------------------------


import cPickle, zlib

def GetHandData():
    return cPickle.loads(zlib.decompress(
'x\xda\xd3\xc8)0\xe4\nV72T\x00!\x05Cu\xae\xc4`u=\x85d\x05\xa7\x9c\xc4\xe4l0O\
\x01\xc8S\xb6t\x06A(\x1f\x0b\xa0\xa9\x8c\x9e\x1e6\x19\xa0\xa8\x1e\x88\xd4C\
\x97\xd1\x83\xe8\x80 \x9c2zh\xa6\xc1\x11X\n\xab\x8c\x02\x8a\x0cD!\x92\x12\
\x98\x8c\x1e\x8a\x8b\xd1d\x14\xf4\x90%\x90LC\xf6\xbf\x1e\xba\xab\x91%\xd0\
\xdc\x86C\x06\xd9m\xe8!\xaa\x87S\x86\x1a1\xa7\x07\x00v\x0f[\x17' ))

def GetHandBitmap():
    return wxBitmapFromXPMData(GetHandData())

def GetHandImage():
    return wxImageFromBitmap(GetHandBitmap())

#----------------------------------------------------------------------
def GetPlusData():
    return cPickle.loads(zlib.decompress(
'x\xda\xd3\xc8)0\xe4\nV72T\x00!\x05Cu\xae\xc4`u=\x85d\x05\xa7\x9c\xc4\xe4l0O\
\x01\xc8S\xb6t\x06A(\x1f\x0b RF\x0f\x08\xb0\xc9@D\xe1r\x08\x19\xb8j=l2`\r\
\xe82HF\xe9a\xc8\xe8\xe9A\x9c@\x8a\x0c\x0e\xd3p\xbb\x00\x8f\xab\xe1>\xd5\xd3\
\xc3\x15:P)l!\n\x91\xc2\x1a\xd6`)\xec\xb1\x00\x92\xc2\x11?\xb8e\x88\x8fSt\
\x19=\x00\x82\x16[\xf7' ))

def GetPlusBitmap():
    return wxBitmapFromXPMData(GetPlusData())

def GetPlusImage():
    return wxImageFromBitmap(GetPlusBitmap())

#----------------------------------------------------------------------
def GetMinusData():
    return cPickle.loads(zlib.decompress(
'x\xda\xd3\xc8)0\xe4\nV72T\x00!\x05Cu\xae\xc4`u=\x85d\x05\xa7\x9c\xc4\xe4l0O\
\x01\xc8S\xb6t\x06A(\x1f\x0b RF\x0f\x08\xb0\xc9@D\xe1r\x08\x19\xb8j=\xa2e\
\x10\x16@\x99\xc82zz\x10\'\x90"\x83\xc34r\xdc\x86\xf0\xa9\x9e\x1e\xae\xd0\
\x81Ja\x0bQ\x88\x14\xd6\xb0\x06Ka\x8f\x05\x90\x14\x8e\xf8\xc1-C|\x9c\xa2\xcb\
\xe8\x01\x00\xed\x0f[\x87' ))

def GetMinusBitmap():
    return wxBitmapFromXPMData(GetMinusData())

def GetMinusImage():
    return wxImageFromBitmap(GetMinusBitmap())

def GetSeparatorData():
    return cPickle.loads(zlib.decompress(
'x\xda\xd3\xc8)0\xe4\nV74P02T0R0T\xe7J\x0cV\xd7SHVPvs3\x00\x020_\x01\xc8\xf7\
\xcb\xcfK\x85r\x14\x14\xf4\xf4@\xe4(\x97$\xae\x1e\x00S\xca=4' ))

def GetSeparatorBitmap():
    return wxBitmapFromXPMData(GetSeparatorData())

def GetSeparatorImage():
    return wxImageFromBitmap(GetSeparatorBitmap())

