#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#----------------------------------------------------------------------------
# Name:         MessageDecoder.py
# Purpose:      A class which contains all the information you need to parse
#               a raw Pushpin message.
#
# Author:       Michael Broxton and Josh Lifton
#
# Created:      A long time ago, in a galaxy far, far away...
# Copyright:    (c) 2004 by MIT Media Laboratory
#
# Comments:
#----------------------------------------------------------------------------

from Globals import *

#
# Pushpin Data (PD) data types.
# 
PD_UCHAR = 0   # One byte,  unsigned char
PD_UINT = 1    # Two bytes, unsigned int
PD_SINT = 2    # Two bytes, signed int
PD_ULONG = 3   # Four bytes, unsigned long
PD_FLOAT = 4   # Four bytes, float

#
# Message type definitions.
#
# These lists provide a key for decoding the content of various
# pushpin messages.  Each triple represents the name of the message
# field and the number of bytes allocated to that field as well as
# the data type of the field, which is used to decode it.
#

# Index of the key to use for storing this info in a dictionary.
ROUTEDREPLY_MESSAGE_DICTKEY_LOCATION = 2
routedReplyMessageDecoder = [ [                'type', 1, PD_UCHAR],
                              [  'requesting node id', 1, PD_UCHAR],
                              [  'responding node id', 1, PD_UCHAR],
                              [            'hopcount', 1, PD_UCHAR],
                              [  'x coordinate ticks', 2, PD_SINT],
                              [  'y coordinate ticks', 2, PD_SINT],
                              [             'ping 1' , 2, PD_UINT],
                              [             'ping 2' , 2, PD_UINT],
                              [             'ping 3' , 2, PD_UINT],
                              [             'ping 4' , 2, PD_UINT],
                              [             'ping 5' , 2, PD_UINT],
                              [             'ping 6' , 2, PD_UINT],
                              [             'ping 7' , 2, PD_UINT],
                              [             'ping 8' , 2, PD_UINT],
                              [             'ping 9' , 2, PD_UINT],
                              [            'ping 10' , 2, PD_UINT]    ]

# Index of the key to use for storing this info in a dictionary.
DUMP_MESSAGE_DICTKEY_LOCATION = 1
ideDumpMessageDecoder = [ [                         'type' , 1, PD_UCHAR],
                          [                           'id' , 1, PD_UCHAR ],
                          [           'ping1LocalDistance' , 2, PD_UINT ],
                          [    'ping1MinimumKnownDistance' , 2, PD_UINT ],
                          [  'ping1MinimumKnownDistanceID' , 1, PD_UCHAR ],
                          [                'ping1AnchorID' , 1, PD_UCHAR ],
                          [  'ping1AnchorDistanceToPinger' , 2, PD_UINT ],
                          [       'ping1AnchorXCoordinate' , 4, PD_FLOAT ],
                          [       'ping1AnchorYCoordinate' , 4, PD_FLOAT ],
                          [ 'ping1MinimumHopCountToAnchor' , 1, PD_UCHAR ],
                          [           'ping2LocalDistance' , 2, PD_UINT ],
                          [    'ping2MinimumKnownDistance' , 2, PD_UINT ],
                          [  'ping2MinimumKnownDistanceID' , 1, PD_UCHAR ],
                          [                'ping2AnchorID' , 1, PD_UCHAR ],
                          [  'ping2AnchorDistanceToPinger' , 2, PD_UINT ],
                          [       'ping2AnchorXCoordinate' , 4, PD_FLOAT ],
                          [       'ping2AnchorYCoordinate' , 4, PD_FLOAT ],
                          [ 'ping2MinimumHopCountToAnchor' , 1, PD_UCHAR ],
                          [           'ping3LocalDistance' , 2, PD_UINT ],
                          [    'ping3MinimumKnownDistance' , 2, PD_UINT ],
                          [  'ping3MinimumKnownDistanceID' , 1, PD_UCHAR ],
                          [                'ping3AnchorID' , 1, PD_UCHAR ],
                          [  'ping3AnchorDistanceToPinger' , 2, PD_UINT ],
                          [       'ping3AnchorXCoordinate' , 4, PD_FLOAT ],
                          [       'ping3AnchorYCoordinate' , 4, PD_FLOAT ],
                          [ 'ping3MinimumHopCountToAnchor' , 1, PD_UCHAR ],
                          [                 'X-coordinate' , 4, PD_FLOAT ],
                          [                 'Y-coordinate' , 4, PD_FLOAT ],
                          [            'numberOfNeighbors' , 1, PD_UCHAR ]]

#
# Using the message type, determine which packet decoder to use.
#
def getDecoderForMessageType(messageType):
    
    if (messageType == ROUTED_REPLY_MESSAGE):
        return MessageDecoder(routedReplyMessageDecoder,
                              ROUTEDREPLY_MESSAGE_DICTKEY_LOCATION)
    elif (messageType == IDE_DUMP_MESSAGE):
        return MessageKey(ideDumpMessageDecoder,
                          DUMP_MESSAGE_DICTKEY_LOCATION)
    else:
        return None


class MessageDecoder:
    """
    The message key class encapsulates the messageDecoder object and the DICTKEY_LOCATION
    information.  The purpose of this class is to create a nice, clean interface for looking up
    information about how to decode a message. 
    """

    def __init__ (self, messageKey, dictionaryKeyLocation):
        self.dictionaryKeyLocation = dictionaryKeyLocation
        self.messageKey = messageKey
        
    #
    # Get the number of fields contained in this packet
    # 
    def getNumberOfFields(self):
        return len(self.messageKey)

    #
    # Return the location in the packet to search for the directory key.
    #
    def getDictionaryKeyLocation(self):
        return self.dictionaryKeyLocation

    #
    # Return the raw message key as a 2-d list.
    #
    def getMessageKey(self):
        return self.messageKey

    #
    # Get the string data for the given field.
    # - 'index' is the zero-based index of the field.
    #
    def getFieldString(self, index): 
        return self.messageKey[index][0]

    #
    # Get the string data for all fields as a list.
    #
    def getFieldStringsAsList(self): 
        fieldList = []
        for x in range( self.getNumberOfFields() ):
            fieldList.append( self.getFieldString(x) )
        return fieldList
        
    #
    # Get the size of the data for the given field.
    # - 'index' is the zero-based index of the field.
    #
    def getFieldSize(self, index): 
        return self.messageKey[index][1]

    #
    # Get the data type of a given field.
    # - 'index' is the zero-based index of the field.
    #
    def getFieldType(self, index):
        return self.messageKey[index][2]
