#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#----------------------------------------------------------------------------
# Name:         PushpinDirectory.py
# Purpose:      A class for managing the data from many Pushpins.
#
# Author:       Michael Broxton and Josh Lifton
#
# Created:      A long time ago, in a galaxy far, far away...
# Copyright:    (c) 2004 by MIT Media Laboratory
#
# Comments:
# - Directories are implemented with python dictionaries.  The key for each
#   dictionary entry is the ID of the node.  The message type and legend of the
#   NodeDirectory are taken from the first packet that is added.
#----------------------------------------------------------------------------

from Globals import *
from NotificationCenter import *

class NodeDirectory (NotificationCenter):
    """
    Meme - Add some more detailed comments here!!!
    """
    
    def __init__(self, controller):
        self.controller = controller

        # Create the empty dictionary
        self.legend = None
        self.directory = {}
        NotificationCenter.__init__(self)

    #
    # Return a list in which each entry corresponds to one field
    # in the decoded packet.
    #
    def getDirectoryLegend(self):
        return self.legend

    def clearDirectory(self) :
        self.directory = {}

    #
    # Return the directory in the form of a dictionary.  The keys of the
    # correspond to the Neighborhood IDs of the nodes.
    #
    def getDirectoryDict(self):
        return self.directory
    
    #
    # Add some data from the content of a Pushpin Packet.
    #
    # If there already exists an entry in the dictionary for the given key, that data is
    # overwritten by the new data from the packet.
    #
    # Incoming packets are arrays of ascii encoded byte strings.  Entries are decoded and
    # cast into the correct type using the directoryKey.
    #
    # Meme - Since python does not support bytes as a numeric literal, all bytes are
    # encoded as integers for now.
    #
    def NotificationCallback(self, packet):
        # The first packet determines the message type and legend for this directory.
        if self.directory == {}:
            self.messageType = packet.getMessageType()
            self.legend = packet.getMessageLegend()

        # All subsequent packets must be of the same message type.
        elif packet.getMessageType() != self.messageType:
            print >> self.controller.pd_stderr, ("Error in NodeDirectory.addDataFromPacket(): ",
                                                 "Wrong packet type.")
            return

        # Extract the dictionary key from the packet
        key = int( packet.getDictionaryKey(), 16 )
        value = packet.parseIntoList()

        # Add or update the dictionary entry
        self.directory[key] = value

        # Meme - For now, print the entry for debugging.
        #self.printDirectory()

        #
        # Pass the notification up the chain to the next listener, using
        # the message type as the notification key.  For more information,
        # see NotificationCenter.py
        #
        self.NotifyListeners(self, self.messageType)


    #
    # Print out the directory for debugging purposes.
    #
    def printDirectory(self):
        for x in self.directory.keys():
            print x , " : " , self.directory[x]

    
    def toFile(self,file):
        def listtoString(list):
            string = ""
            for word in list:
                if string == "":
                    string = str(word)
                else:
                    string = string+ "\t"+ str(word)
            return string
        
        if self.legend == None:
            print >> self.controller.pd_stderr, "Dongle All Data before printing to File"
        else:
            legend = "Neighborhood ID\t"+listtoString(self.legend)

            dir = ""
            for x in self.directory.keys():
                dir = dir + str(x)+"\t"+listtoString(self.directory[x])+"\n"

            string = legend + "\n\n" + dir
            try:
                f = open(file, "w") 
            except IOError,detail:
                print str(detail)
            else:
                f.write(string)
