#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#----------------------------------------------------------------------------
# Name:         ListPanel.py
# Purpose:      This panel lists each of the known Pushpin nodes along with some
#               of their data.  This panel uses the wxListCtrl class to manage the
#               listing itself.  Two additional 'listmix' classes are used for
#               sorting the list and autosizing the list columns respectively.
#
#               Most of this code is from the wxPython demo application.
#
# Author:       Michael Broxton, Josh Lifton, and Daniel Alejandro Benitez
#
# Created:      A long time ago, in a galaxy far, far away...
# Copyright:    (c) 2004 by MIT Media Laboratory
#----------------------------------------------------------------------------

from time import localtime
import wx
import wx.lib.mixins.listctrl  as  listmix
from Globals import *

LIST_PANEL_NAME = "Listing"
BUTTON_BAR_HEIGHT = 30
ID_SAVE_LIST = wx.NewId()
ID_CLEAR_LIST = wx.NewId()
ID_GET_ALL_DATA = wx.NewId()


class PushpinListCtrl(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
    def __init__(self, parent, ID, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):
        wx.ListCtrl.__init__(self, parent, ID, pos, size, style)
        listmix.ListCtrlAutoWidthMixin.__init__(self)

class ListPanel(wx.Panel, listmix.ColumnSorterMixin):
    def __init__(self, parent, controller, menubar):
        self.controller = controller
        self.menubar = menubar
        wx.Panel.__init__(self, parent, -1, style=wx.WANTS_CHARS)
        
        #
        # Initialize some variables that are specific to the list panel.
        #
        self.legend = None
        self.itemDataMap = None
        self.currentItem = None

        #
        # create the actual ListCtrl object for displaying Pushpin Statistics.
        #
        self.list = PushpinListCtrl(self, wx.NewId(),
                                    style=wx.LC_REPORT 
                                    | wx.SUNKEN_BORDER
                                    | wx.LC_EDIT_LABELS
                                    # | wxLC_NO_HEADER
                                    # | wx.LC_VRULES | wx.LC_HRULES
                                    )

        #
        # Go ahead and populate the list.
        #
        self.PopulateList()

        #
        # Bind some events to the list.
        # Important Note:  We must add these event handler bindings
        # before we initialize the ColumnSorterMixin parent of this
        # class.  This ensures that, during an OnColClick() event,
        # the colorListForReadability() is called *after* the column
        # sorting is handled.
        #
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.list)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemDeselected, self.list)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated, self.list)
        self.Bind(wx.EVT_LIST_DELETE_ITEM, self.OnItemDelete, self.list)
        self.Bind(wx.EVT_LIST_COL_CLICK, self.OnColClick, self.list)
        self.Bind(wx.EVT_LIST_COL_RIGHT_CLICK, self.OnColRightClick, self.list)
        self.Bind(wx.EVT_LIST_COL_BEGIN_DRAG, self.OnColBeginDrag, self.list)
        self.Bind(wx.EVT_LIST_COL_DRAGGING, self.OnColDragging, self.list)
        self.Bind(wx.EVT_LIST_COL_END_DRAG, self.OnColEndDrag, self.list)
        self.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.OnBeginEdit, self.list)
        self.list.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)
        self.list.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        # for wxMSW
        self.list.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)
        # for wxGTK
        self.list.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)

        #
        # Now that the list exists we can init the other base class,
        # which is used to help us sort the data in the list.
        # See wxPython/lib/mixins/listctrl.py
        #
        listmix.ColumnSorterMixin.__init__(self, 20)
        # self.SortListItems(0, True)


        #
        # Create some buttons to control some stuff.
        #
        self.saveButton = wx.Button(self, ID_SAVE_LIST, "Save Listing")
        self.Bind(wx.EVT_BUTTON, self.HandleButtonAction)
        self.resetButton = wx.Button(self, ID_CLEAR_LIST, "Clear")
        self.Bind(wx.EVT_BUTTON, self.HandleButtonAction)
        self.getAllDataButton = wx.Button(self, ID_GET_ALL_DATA, "Collect Data")
        self.Bind(wx.EVT_BUTTON, self.HandleButtonAction)

        #
        # Register with the notification center that generates the Node Directory.
        #
        controller.getNodeDirectoryNotificationCenter().RegisterListener(self,'ALL')


    #
    # This routine is used to determine what string should be used to label
    # this notebook tab.
    #
    def Name(self):
        return LIST_PANEL_NAME

    #
    # Callback function for use with Notification Centers.  Any
    # notification center this object is registered as a listener with
    # will notify this object when new data becomes available.  For
    # more information, see NotificationCenter.py.
    #
    def NotificationCallback(self, nodeDirectory):
        self.legend = nodeDirectory.getDirectoryLegend()
        self.nodeDirectory = nodeDirectory # Need access to this so as to clear the directory.
        self.itemDataMap = nodeDirectory.getDirectoryDict()
        self.PopulateList()

    #
    # Populate the list using the data provided.
    #
    def PopulateList(self):
        self.list.ClearAll()

        self.widthOfListColumns = [ 120, 40, 120, 120, 60, 175, 175 ]
        self.extraListLegendText = [ '', '', '','', ' ( meters ) ', ' ( meters ) ' ]
 
        #
        # If the item data map and the legend have not yet been specified,
        # we don't do anything.
        #
        if (self.itemDataMap == None) or (self.legend == None) :
            return
        
        # Add the columns from the data map legend.  The first column
        # will be the dictionary key, which isn't included in the
        # legend.
        numberOfColumns = len(self.legend) + 1 
        self.list.InsertColumn(0, 'Neighborhood ID')
        for z in range( len(self.legend) ):
            if ( z < len( self.extraListLegendText ) ) :
                self.list.InsertColumn( z + 1, self.legend[z] + self.extraListLegendText[z] )
            else :
                self.list.InsertColumn( z + 1, self.legend[z] )

        #
        # Now that the legend has been printed along the top of the
        # columns, we add the data itself.
        #
        items = self.itemDataMap.items()
        for x in range(len(items)): 
            key, data = items[x]
            # The key usually contains the node ID.  We insert this as a hex string.
            self.list.InsertStringItem(x, str("%02X" % key)) 
            for y in range( len(data) ):
                #
                # MEME - This is a rather messy hack to convert
                # Pushpin clock ticks to meters in the listing.
                #
                if ( ( y < len ( self.extraListLegendText ) ) and ( self.extraListLegendText[y] == ' ( meters ) ' ) ) :
                    self.list.SetStringItem( x, y + 1, (data[y] +
                                                        ' ( ' +
                                                        str( self.hexToSignedInt( data[y] ) / CLOCK_TICKS_PER_METER ) +
                                                        ' ) ') )
                else :
                    self.list.SetStringItem( x, y + 1, data[y] )
            self.list.SetItemData(x, key)

        for w in range ( numberOfColumns ) :
            if ( w < len( self.widthOfListColumns ) ) :
                 self.list.SetColumnWidth( w, self.widthOfListColumns[w] )
            else :
                 self.list.SetColumnWidth( w, 120)

        # Color the list for readability
        self.colorListForReadability()


    #
    # This method is used by the ColumnSorterMixin, see
    # wxPython/lib/mixins/listctrl.py
    #
    def GetListCtrl(self):
        return self.list

    def getColumnText(self, index, col):
        item = self.list.GetItem(index, col)
        return item.GetText()

    #
    # Color alternating rows with a light grey to enhance readability.
    #
    def colorListForReadability(self):
        COLORLIST = [wx.Colour(230,230,230),wx.Colour(255, 255, 255)]
        for x in range( self.list.GetItemCount() ):
            item = self.list.GetItem(x)
            item.SetBackgroundColour(COLORLIST[x%2])
            self.list.SetItem(item)

    #
    # Utility function for printing hex data in a human readable format.
    #
    def hexToSignedInt(self, hexString) :
        hexString = hexString.upper()
        if int(hexString,16) > 0x7FFF :
            twosComp = ''
            hexString = str('%04X' % (int(hexString,16) - 1))
            for c in hexString :
                twosComp = twosComp + str('%01X' % abs(int(c,16) - 15))
            return ( -1*int(twosComp,16) )
        return int(hexString,16)



    # -------------------------------------------------------------------
    # EVENT HANDLERS

    #
    # This function handles button presses in the listpanel GUI.  Its
    # main purpose is to save a tab delimitted data file to disk when
    # the "Save List" button is pressed in the GUI.
    #
    def HandleButtonAction(self, event):
        if event.GetId() == ID_SAVE_LIST:
            defaultFilename = ("node-directory__" + str(localtime()[0]) + "-" +
                        str(localtime()[1]) + "-" +
                        str(localtime()[2]) + "__" +
                        str(localtime()[3]) + "-" +
                        str(localtime()[4]) + ".txt")
#            saveDialog = wx.FileDialog(self, "Choose a destination...", "",
#                                       defaultFilename, "*.*", wx.SAVE)
#            saveDialog.ShowModal()
            if defaultFilename != "":
#                print "Saving file: " + saveDialog.GetPath()
                print "Saving file: " + defaultFilename;
                  #
                # If the item data map and the legend have not yet been specified,
                # we don't do anything.
                #
                if (self.itemDataMap == None) or (self.legend == None) :
                    print >> self.controller.pd_stderr, "Warning: No list data available.  Export to file cancelled."
                else:
                    try:
                        f = open(defaultFilename, 'w')

                        # Write the legend to disk
                        textLegend = map(lambda x: x + '\t', self.legend)
                        f.write("Neighborhood ID\t")
                        for x in textLegend:
                            f.write(x)

                        # Write the data to disk
                        dataRows = self.itemDataMap.items()
                        for x in range(len(dataRows)):
                            f.write('\n')
                            key, data = dataRows[x]
                            f.write(str("%02X\t" % key))
                            for y in data:
                                f.write(str(y) + "\t")
                        
                    except:
                        print >> self.controller.pd_stderr, "Error: Could not save file."
                        print >> self.controller.pd_stderr, "\t", sys.exc_info()[0], "\n\t", sys.exc_info()[1].args[0]

                    
            else:
                # User cancelled the save operation.
                print "User Cancelled..."
                pass

        elif event.GetId() == ID_CLEAR_LIST:
            ## Clear the GUI list
            self.list.ClearAll()
            self.controller.getNodeDirectoryNotificationCenter().UnregisterListener(self,'ALL')
            self.controller.resetNodeDirectory()
            self.controller.getNodeDirectoryNotificationCenter().RegisterListener(self,'ALL')
            # meme : just a kludge
            self.nodeDirectory.clearDirectory()
            self.itemDataMap = None

        elif event.GetId() == ID_GET_ALL_DATA :
            ## Try to collect all data from nodes not yet collected from.
            self.getAllDataButton.Disable()
            f = open('NeighborhoodIDs.dat', 'r')
            allIds = [x.strip().upper() for x in f.readlines()]
            f.close()
            allIds.sort()
            noDataIds = [x for x in allIds]
            if self.itemDataMap :
                yesDataIds = [str('%02X' % x) for x in self.itemDataMap.keys()]
                for id in yesDataIds :
                    noDataIds.remove(id)
            print('Need data for ' + str(noDataIds))
            for id in noDataIds :
                self.controller.dongleDataLink.sendQueryPacket(int(id,16))
                self.controller.Yield()
            newLabel = 'Collect Data'
            if self.itemDataMap :
                yesDataIds = [str('%02X' % x) for x in self.itemDataMap.keys()]
                yesDataIds.sort()
                if yesDataIds != allIds :
                    newLabel = 'Collect Remaining Data'
            self.getAllDataButton.SetLabel(newLabel)
            self.getAllDataButton.Enable()

    #
    # this routine is called when this notebook panel becomes active.
    #
    def OnActivate(self):
        pass

    #
    # This routine is called when this notebook panel becomes inactive.
    #
    def OnDeactivate(self):
        pass

    # Used by the ColumnSorterMixin, see wxPython/lib/mixins/listctrl.py
    #    def GetSortImages(self):
    #        return (self.sm_dn, self.sm_up)

    def OnRightDown(self, event):
        self.x = event.GetX()
        self.y = event.GetY()
        print "x, y = %s\n" % str((self.x, self.y)) 
        item, flags = self.list.HitTest((self.x, self.y))

        if flags & wx.LIST_HITTEST_ONITEM:
            self.list.Select(item)
        event.Skip()

    def OnItemSelected(self, event):
        self.currentItem = event.m_itemIndex
        print ("OnItemSelected: %s, %s, %s, %s\n" %
                           (self.currentItem,
                            self.list.GetItemText(self.currentItem),
                            self.getColumnText(self.currentItem, 1),
                            self.getColumnText(self.currentItem, 2)))
        event.Skip()


    def OnItemDeselected(self, evt):
        item = evt.GetItem()
        print "OnItemDeselected: %d" % evt.m_itemIndex 

    def OnItemActivated(self, event):
        self.currentItem = event.m_itemIndex
        print ("OnItemActivated: %s\nTopItem: %s" %
                           (self.list.GetItemText(self.currentItem), self.list.GetTopItem()))

    def OnBeginEdit(self, event):
        print "OnBeginEdit"
        event.Allow()

    def OnItemDelete(self, event):
        print "OnItemDelete\n"

    def OnColClick(self, event):
        print "OnColClick: %d\n" % event.GetColumn()
        # Color the list for readability
        self.colorListForReadability()
#        event.Skip()

    def OnColRightClick(self, event):
        item = self.list.GetColumn(event.GetColumn())
        print ("OnColRightClick: %d %s\n" %
                           (event.GetColumn(), (item.GetText(), item.GetAlign(),
                                                item.GetWidth(), item.GetImage())))

    def OnColBeginDrag(self, event):
        print "OnColBeginDrag\n"
        ## Show how to not allow a column to be resized
        #if event.GetColumn() == 0:
        #    event.Veto()


    def OnColDragging(self, event):
        print ("OnColDragging\n")

    def OnColEndDrag(self, event):
        print ("OnColEndDrag\n")

    def OnDoubleClick(self, event):
        if self.currentItem!=None:
            print("OnDoubleClick item %s\n" % self.list.GetItemText(self.currentItem))
            event.Skip()

    def OnRightClick(self, event):
        if self.currentItem!=None:
            print ("OnRightClick %s\n" % self.list.GetItemText(self.currentItem))

        # only do this part the first time so the events are only bound once
        if not hasattr(self, "popupID1"):
            self.popupID1 = wx.NewId()
            self.popupID2 = wx.NewId()
            self.popupID3 = wx.NewId()
            self.popupID4 = wx.NewId()
            self.popupID5 = wx.NewId()
            self.popupID6 = wx.NewId()

            self.Bind(wx.EVT_MENU, self.OnPopupOne, id=self.popupID1)
            self.Bind(wx.EVT_MENU, self.OnPopupTwo, id=self.popupID2)
            self.Bind(wx.EVT_MENU, self.OnPopupThree, id=self.popupID3)
            self.Bind(wx.EVT_MENU, self.OnPopupFour, id=self.popupID4)
            self.Bind(wx.EVT_MENU, self.OnPopupFive, id=self.popupID5)
            self.Bind(wx.EVT_MENU, self.OnPopupSix, id=self.popupID6)

        # make a menu
        menu = wx.Menu()
        # add some items
        menu.Append(self.popupID1, "FindItem tests")
        menu.Append(self.popupID2, "Iterate Selected")
        menu.Append(self.popupID3, "ClearAll and repopulate")
        menu.Append(self.popupID4, "DeleteAllItems")
        menu.Append(self.popupID5, "GetItem")
        menu.Append(self.popupID6, "Edit")

        # Popup the menu.  If an item is selected then its handler
        # will be called before PopupMenu returns.
        self.PopupMenu(menu, (self.x, self.y))
        menu.Destroy()


    def OnPopupOne(self, event):
        print ("Popup one\n")
        print "FindItem:", self.list.FindItem(-1, "Roxette")
        print "FindItemData:", self.list.FindItemData(-1, 11)

    def OnPopupTwo(self, event):
        print ("Selected items:\n")
        index = self.list.GetFirstSelected()

        while index != -1:
            print ("      %s: %s\n" % (self.list.GetItemText(index), self.getColumnText(index, 1)))
            index = self.list.GetNextSelected(index)

    def OnPopupThree(self, event):
        print ("Popup three\n")
        self.list.ClearAll()
        wx.CallAfter(self.PopulateList)

    def OnPopupFour(self, event):
        self.list.DeleteAllItems()

    def OnPopupFive(self, event):
        item = self.list.GetItem(self.currentItem)
        print item.m_text, item.m_itemId, self.list.GetItemData(self.currentItem)

    def OnPopupSix(self, event):
        self.list.EditLabel(self.currentItem)

    def OnSize(self, event):
        w,h = self.GetClientSizeTuple()
        if h > BUTTON_BAR_HEIGHT:
            self.list.SetDimensions(0, 0, w, h - BUTTON_BAR_HEIGHT)
            self.saveButton.SetDimensions(10, (h - BUTTON_BAR_HEIGHT) + 5, 150, 25)
            self.resetButton.SetDimensions(175, (h - BUTTON_BAR_HEIGHT) + 5, 150, 25)
            self.getAllDataButton.SetDimensions(340, (h - BUTTON_BAR_HEIGHT) + 5, 150, 25)


def runPanel (parent, controller, menubar):
    return ListPanel(parent, controller, menubar)

