#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#----------------------------------------------------------------------------
# Name:         PlotPanel.py
# Purpose:      This panel plots the current positions of localized Pushpin
#               nodes.  It depends on the SciPy library for scientific computing,
#               which in turn has many of its own dependencies.  For instructions
#               on installing SciPy, check www.scipy.org.
#
#               For specific MacOS X installation instructions, refer to:
#               http://www.scipy.org/documentation/Members/fonnesbeck/osx_build.txt
#               
# Author:       Michael Broxton, Josh Lifton, & Daniel Alejandro Benitez
#
# Created:      A long time ago, in a galaxy far, far away...
# Copyright:    (c) 2004 by MIT Media Laboratory
#----------------------------------------------------------------------------

import wx
import wx.lib.plot as plot
from Globals import *
from GraphicalInterface import *
from ListPanel import *

    
class PlotPanel (wx.Panel):
    def __init__(self, parent, controller): 
        wx.Panel.__init__(self, parent)
        
        self.parent = parent
        self.controller = controller
        self.itemDataMap = None  # Initialize with an empty map.
        self.legend = None

        self.splitter = ProportionalSplitter( self, proportion = 0.50 )

        self.splitter2 = ProportionalSplitter( self.splitter, proportion = 0.50 )
        self.logPanel = LogPanel(self.splitter2)
        self.module = __import__("ListPanel", globals())
        self.listPanel = self.module.runPanel(self.splitter2,controller)
        self.splitter2.SplitVertically(self.logPanel,self.listPanel)

        self.plotFrame = plot.PlotCanvas(self.splitter)
        self.plotFrame.Bind(wx.EVT_LEFT_DOWN, self.OnMouseLeftDown)

        self.splitter.SplitVertically(self.plotFrame, self.splitter2)
        
##        self.splitter.SplitVertically(self.plotFrame, self.logPanel)
        
        # Use some sizers to see layout options
        self.sizer=wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.splitter, 1, wx.EXPAND)
        
        #Layout sizers
        self.SetSizer(self.sizer)
        self.SetAutoLayout(1)


        self.resetDefaults()
        self.plotFrame.SetFont(wx.Font(10,wx.SCRIPT,wx.NORMAL,wx.NORMAL))
        self.plotFrame.SetFontSizeAxis(20)
        self.plotFrame.SetFontSizeLegend(12)
        self.plotFrame.SetXSpec('min')
        self.plotFrame.SetYSpec('none')

        self.PopulateGraphList()

        self.plotFrame.Bind(wx.EVT_RIGHT_DOWN, self.OnMouseRightDown)
        self.plotFrame.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        
#        markerList= ['circle', 'dot', 'square', 'triangle', 'triangle_down',
#                     'cross', 'plus', 'circle']
#        m=[]
#        for i in range(len(markerList)):
#            m.append(plot.PolyMarker([(2*i+.5,i+.5)], legend=markerList[i], colour='blue',
#                                marker=markerList[i]))
#            self.plotFrame.Draw(plot.PlotGraphics(m, "Selection of Markers", "Minimal Axis", "No Axis"))


    def setNodeDirectory(self, directory):
        self.legend = directory.getDirectoryLegend()
        self.itemDataMap = directory.getDirectoryDict()

    def hexToSignedInt(self, hexString) :
        hexString = hexString.upper()
        if int(hexString,16) > 0x7FFF :
            twosComp = ''
            hexString = str('%04X' % (int(hexString,16) - 1))
            for c in hexString :
                twosComp = twosComp + str('%01X' % abs(int(c,16) - 15))
            return -1*int(twosComp,16)
        return int(hexString,16)

    def mouseClickOnNode( self, eventXY ) :
        eventX, eventY = self.plotFrame.GetXY(eventXY)
        bestFit = ( None, 1 )
        key = self.xyCoor.keys()
        
        for i in range( 0, len( key ) ) :
            nodeX, nodeY = self.xyCoor[ key[i] ]
            distSquared = ( pow( ( nodeX - eventX ), 2 ) + pow( ( nodeY - eventY ), 2) )
            if ( bestFit[1] > distSquared ) :
                bestFit = ( key[i], distSquared )

        if ( bestFit[1] > 0.0025 ) :
            print ( 'You are not clicking on a node' )
            return
        else :
            print ( 'You are clicking on node ' + str( bestFit[0] ) + ' at (%.4f, %.4f).' % ( nodeX, nodeY ) )
            return bestFit[0]

    def PopulateGraphList(self):
        if (self.itemDataMap == None) :
            return
        if  (self.legend == None) :
            return

        items = self.itemDataMap.items()
        
        self.nodes=[]
        self.xyCoor = {}
#        self.nodes.append(plot.PolyMarker([(10,10)], legend='test', colour='red', marker='circle'))
#        self.plotFrame.Draw(plot.PlotGraphics(self.nodes, "Selection of Markers", "Minimal Axis", "No Axis"))
        for i in range(0,len(items)):
            key, data = items[i]
            x_coor = ( self.hexToSignedInt( data[4] ) / CLOCK_TICKS_PER_METER )
            y_coor = ( self.hexToSignedInt( data[5] ) / CLOCK_TICKS_PER_METER )
            self.xyCoor[str("%02X" % key)] = ( x_coor, y_coor )
            self.nodes.append(plot.PolyMarker([(x_coor, y_coor)], legend=key, colour='blue',
                                marker='circle', fillstyle=wx.TRANSPARENT))
            self.plotFrame.Draw(plot.PlotGraphics(self.nodes, "Pushpin Node Localization",
                                                  "X-Coordinate Axis", "Y-Coordinate Axis"),
                                xAxis = ( -1.0, 1.0 ), yAxis = ( -1.0, 1.0 ))

        print 'I am the x,y coordinate dictionary!!! ' + str( self.xyCoor )
        
    def setDataMap(self, dataMap):
        self.itemDataMap = dataMap

    def OnMouseLeftDown(self,event):
        print "Left Mouse Down at Point: (%.4f, %.4f)" % self.plotFrame.GetXY(event)
        selectedNode = self.mouseClickOnNode( event )

        if ( selectedNode == None ) :
            pass
        else :
            x, y = self.xyCoor[ selectedNode ]
            self.nodes.append( plot.PolyMarker( [(x,y)], legend=selectedNode, colour='red', marker='circle' ) )
            self.plotFrame.Draw( plot.PlotGraphics( self.nodes, "Pushpin Node Localization", "X-Coordinate Axis", "Y-Coordinate Axis" ), xAxis = ( -1.0, 1.0 ), yAxis = ( -1.0, 1.0 ) )
            self.nodes.pop()

        event.Skip()

    def OnMouseRightDown(self,event):
        #Recenters the plot with the mouse coordinate being the new center 
        if self.plotFrame.GetEnableZoom():
            X,Y = self.plotFrame.GetXY(event)
            self.plotFrame.Zoom( (X,Y), (1,1))

            
    def OnFilePageSetup(self, event):
        self.plotFrame.PageSetup()
        
    def OnFilePrintPreview(self, event):
        self.plotFrame.PrintPreview()
        
    def OnFilePrint(self, event):
        self.plotFrame.Printout()
        
    def OnSaveFile(self, event):
        self.plotFrame.SaveFile()

    def OnPlotRedraw(self,event):
        self.plotFrame.Redraw()

    def OnPlotClear(self,event):
        self.plotFrame.Clear()
        
    def OnPlotScale(self, event):
        if self.plotFrame.last_draw != None:
            graphics, xAxis, yAxis= self.plotFrame.last_draw
            self.plotFrame.Draw(graphics,(1,3.05),(0,1))

    def OnEnableZoom(self, event):
        self.plotFrame.SetEnableZoom(event.IsChecked())
        
    def OnEnableGrid(self, event):
        self.plotFrame.SetEnableGrid(event.IsChecked())
        
    def OnEnableLegend(self, event):
        self.plotFrame.SetEnableLegend(event.IsChecked())

    def OnScrUp(self, event):
        self.plotFrame.ScrollUp(1)
        
    def OnScrRt(self,event):
        self.plotFrame.ScrollRight(2)

    def OnReset(self,event):
        self.plotFrame.Reset()

    def resetDefaults(self):
        """Just to reset the fonts back to the PlotCanvas defaults"""
        self.plotFrame.SetFont(wx.Font(10,wx.SWISS,wx.NORMAL,wx.NORMAL))
        self.plotFrame.SetFontSizeAxis(10)
        self.plotFrame.SetFontSizeLegend(7)
        self.plotFrame.SetXSpec('auto')
        self.plotFrame.SetYSpec('auto')

    def OnAutoZoom(self,event):
        maxX = self.plotFrame.GetXMaxRange()[1]
        maxY = self.plotFrame.GetYMaxRange()[1]
        curX = self.plotFrame.GetXCurrentRange()[1]
        curY = self.plotFrame.GetYCurrentRange()[1]

        self.plotFrame.SetXSpec("auto")
        self.plotFrame.SetYSpec("auto")
##        self.plotFrame.Zoom( (0,0), ((maxX/curX),(maxY/curY)) )
##        print "MaxX is: "+str(maxX)+" CurX is: "+str(curX)
##        print "MaxY is: "+str(maxY)+" CurY is: "+str(curY)
##
    
    def OnKeyDown(self,event):
        minX,maxX = self.plotFrame.GetXCurrentRange()
        minY,maxY = self.plotFrame.GetYCurrentRange()
        center = ( (maxX+minX)/2, (maxY+minY)/2 )
        zoomInRatio = .5
        zoomOutRatio = 2
        
        if event.GetKeyCode() == 392:
            if self.plotFrame.GetEnableZoom():
                self.plotFrame.Zoom(center, (zoomInRatio,zoomInRatio))
        elif event.GetKeyCode() == 394:
            if self.plotFrame.GetEnableZoom():
                self.plotFrame.Zoom(center, (zoomOutRatio,zoomOutRatio))

            
def runPanel (parent, controller):
    return PlotPanel(parent, controller)
