#!/bin/env python

#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#----------------------------------------------------------------------------
# Name:         PushpinDebugger.py
# Purpose:      A graphical debugger for the Pushpin sensor network which uses
#               the WXPython graphics and PySerial serial packages.  
#
# Author:       Michael Broxton and Josh Lifton
#
# Created:      A long time ago, in a galaxy far, far away...
# Copyright:    (c) 2004 by MIT Media Laboratory
#----------------------------------------------------------------------------

## import all of the wxPython GUI package
from wxPython.wx import *
import sys

from Globals import *
from GraphicalInterface import *
from PushpinOS import PushpinOS
from DataLink import DataLink
from NodeDirectory import *

#---------------------------------------------------------------------------
# Every wxWindows application must have a class derived from wxApp.
#
# This particular application is built around the
# Model-View-Controller abstraction for graphical user interfaces.
# This means that in addition to creating objects for the graphical
# user interface and the software itself, this class acts as a
# controller that is the intermediary -- the abstraction barrier --
# between software and the graphical interface.
#
# Thus, beyond the initialization routine ( OnInit() ) that gets everything
# going, this class will have two types of methods: 1) Methods that are called
# when events occur in the GUI that need to interact with software components
# in the back end (actions), and 2) Methods that are called by the software back end
# to update the state of the GUI (outlets).
#---------------------------------------------------------------------------
class PushpinDebuggerApp(wxApp):

    #
    # ====================== Actions ============================
    #
    def HandleToolbarAction(self,event):
        #
        # Meme - There may be a bug here -- You would think that you would
        # have to negate the current state of the button to toggle it on or
        # off. Instead, it would appear that if you set the toggle state to
        # its current state, it actually toggles states!
        #
        #        nextState = self.mainWindow.ToolBar.GetToolState(event.GetId())
        #        self.mainWindow.ToolBar.ToggleTool(event.GetId(), nextState)
        if event.GetId() == ID_OS_COMPILE:
            print 'Compiling and loading OS...'
            self.pushpinOS = PushpinOS(self)
            self.pushpinOS.printAttributes()
            
        elif event.GetId() == ID_OS_SEND:
            if self.spotlightDataLink.connectSerialPort(SPOTLIGHT):
                if self.pushpinOS:
                    self.spotlightDataLink.sendOSPacket(self.pushpinOS)
                else:
                    print >> self.pd_stderr, "No Pushpin OS available.  Compile and Load the Pushpin OS first."
                  
        elif event.GetId() == ID_SPOTLIGHT_CONNECT:
            self.spotlightDataLink.connectSerialPort(SPOTLIGHT)
                
        elif event.GetId() == ID_SPOTLIGHT_DISCONNECT:
            self.spotlightDataLink.disconnectSerialPort(SPOTLIGHT)
                        
        elif event.GetId() == ID_SPOTLIGHT_WAIT:
            if self.spotlightDataLink.connectSerialPort(SPOTLIGHT):
                self.spotlightDataLink.sendWaitPacket()
            
        elif event.GetId() == ID_SPOTLIGHT_START:
            if self.spotlightDataLink.connectSerialPort(SPOTLIGHT):
                self.spotlightDataLink.sendStartPacket()
            
        elif event.GetId() == ID_SPOTLIGHT_SILENCE:
            if self.spotlightDataLink.connectSerialPort(SPOTLIGHT):
                self.spotlightDataLink.sendSilencePacket()

        elif event.GetId() == ID_DONGLE_CONNECT:
            self.dongleDataLink.connectSerialPort(DONGLE)
                        
        elif event.GetId() == ID_DONGLE_DISCONNECT:
            self.dongleDataLink.disconnectSerialPort(DONGLE)
              
        elif event.GetId() == ID_DONGLE_ALL_DATA:
            if self.dongleDataLink.connectSerialPort(DONGLE):
                for x in range( int('42',16) ):
                    self.dongleDataLink.sendQueryPacket( x )
                    wxYield()
                
        elif event.GetId() == ID_DONGLE_QUERY:
            if self.dongleDataLink.connectSerialPort(DONGLE):
                nodeIdStr = self.mainWindow.queryTextCtrl.GetLineText(0)
                if nodeIdStr == "" :
                    print >> self.pd_stderr, "Please choose a node to query."
                elif len(nodeIdStr) > 2:
                    print >> self.pd_stderr, "Invalid node ID."
                else:
                    self.dongleDataLink.sendQueryPacket( int(nodeIdStr,16) )
                   
        elif event.GetId() == ID_DONGLE_MONITOR:
            if self.dongleDataLink.connectSerialPort(DONGLE):
                pass  # Meme - Yet to be implemented

        
    def HandleButtonAction(self,event):
        pass  # There are currently no button events to handle.
    
    #
    # This function handles various menu options in the GUI.
    #
    def HandleMenuAction(self, event):
        if event.GetId() == ID_FILE_ABOUT:
            self.mainWindow.AboutBox()

        elif event.GetId() == ID_FILE_EXIT:
            self.mainWindow.Close(true)

        elif event.GetId() == ID_BUILD_WAIT:
            if self.connectSerialPort(self.spotlightDataLink, SPOTLIGHT):
                self.spotlightDataLink.sendWaitPacket()

        elif event.GetId() == ID_BUILD_START:
            if self.connectSerialPort(self.spotlightDataLink, SPOTLIGHT):
                self.spotlightDataLink.sendStartPacket()

        elif event.GetId() == ID_BUILD_COMPILE:
            print 'Compiling and loading OS...'
            self.pushpinOS = PushpinOS(self)
            self.pushpinOS.printAttributes()

        elif event.GetId() == ID_BUILD_SEND:
            if self.connectSerialPort(self.spotlightDataLink, SPOTLIGHT):
                if self.pushpinOS:  
                    self.spotlightDataLink.sendOSPacket(self.pushpinOS)
                else:
                    print >> self.pd_stderr, "No Pushpin OS available.  Compile and Load the Pushpin OS first."

        elif event.GetId() == ID_BUILD_INFO:
            if self.pushpinOS != None :
                self.pushpinOS.printAttributes()
            else:
                print >> self.pd_stderr , 'Pushpin OS has not been loaded! '

        elif event.GetId() == ID_DEBUG_SILENCE:
            if self.connectSerialPort(self.dongleDataLink, DONGLE):
                self.dongleDataLink.sendSilencePacket()

        elif event.GetId() == ID_DEBUG_QUERY:
            if self.connectSerialPort(self.dongleDataLink, DONGLE):
                pass   # Not Yet Implemented

        elif event.GetId() == ID_DEBUG_DUMP:
            if self.connectSerialPort(self.dongleDataLink, DONGLE):
                self.dongleDataLink.sendIDERequestPacket()
            
        elif event.GetId() == ID_OPTIONS_SPOTLIGHT_PORT1:
            if self.connectSerialPort(self.spotlightDataLink, SPOTLIGHT):
                self.spotlightDataLink.setSerialPort(0)

        elif event.GetId() == ID_OPTIONS_SPOTLIGHT_PORT2:
            if self.connectSerialPort(self.spotlightDataLink, SPOTLIGHT):
                self.spotlightDataLink.setSerialPort(1)

        elif event.GetId() == ID_OPTIONS_IRSERIALDONGLE_PORT1:
            if self.connectSerialPort(self.dongleDataLink, DONGLE):
                self.dongleDataLink.setSerialPort(0)

        elif event.GetId() == ID_OPTIONS_IRSERIALDONGLE_PORT2:
            if self.connectSerialPort(self.dongleDataLink, DONGLE):
                self.dongleDataLink.setSerialPort(1)
                
        elif event.GetId() == ID_PLOT_ENABLEZOOM:
            self.mainWindow.graphPanel.OnEnableZoom(event)

        elif event.GetId() == ID_PLOT_AUTOZOOM:
            self.mainWindow.graphPanel.OnAutoZoom(event)

        elif event.GetId() == ID_PLOT_PAGESETUP:
            self.mainWindow.graphPanel.OnFilePageSetup(event)
            
        elif event.GetId() == ID_PLOT_PRINTPREVIEW:
            self.mainWindow.graphPanel.OnFilePrintPreview(event)

        elif event.GetId() == ID_PLOT_PRINT:
            self.mainWindow.graphPanel.OnFilePrint(event)

        elif event.GetId() == ID_PLOT_SAVE:
            self.mainWindow.graphPanel.OnSaveFile(event)

        elif event.GetId() == ID_PRINT_NODES:
            self.localizationDirectory.toFile("NodeDirectory.txt")



    def getSerialNotificationCenter(self):
        return self.dongleDataLink

    def getNodeDirectoryNotificationCenter(self):
        return self.localizationDirectory

    def resetNodeDirectory(self):
        self.dongleDataLink.UnregisterListener(self.localizationDirectory, ROUTED_REPLY_MESSAGE)
        self.localizationDirectory = NodeDirectory(self)
        self.dongleDataLink.RegisterListener(self.localizationDirectory, ROUTED_REPLY_MESSAGE)

    #
    # ====================== Outlets ============================
    #
    def SetStatusBarText(self, string):
        self.mainWindow.StatusBar.SetStatusText(string, 0)

    def AppendTextToLog(self, string):
        self.mainWindow.splitter.logPanel.AppendText(string + "\n")

    def AppendErrorToLog(self, string):
        self.mainWindow.splitter.logPanel.AppendText("Error: " + string + "\n")

#    def UpdateGraphPanel(self):
#        self.mainWindow.graphPanel.setNodeDirectory( self.localizationDirectory)
#        self.mainWindow.graphPanel.PopulateGraphList()

    def UpdateOSUploadProgess(self, percent):
        self.mainWindow.updateStatusDialog(percent)
        wxYield()

    def SetSpotlightButtonStatus(self, status):
        self.mainWindow.setSpotlightGUI(status)

    def SetDongleButtonStatus(self, status):
        self.mainWindow.setDongleGUI(status)

    #
    # =================== Initialization =========================
    # wxPython calls this method to initialize the application.
    # This is where the controller object creates the model and
    # view objects.
    #
    def OnInit(self):

        # Create an instance of our customized Frame class - the GUI.
        self.mainWindow = MainWindow(self, -1, "PushpinDebugger")
        self.SetTopWindow(self.mainWindow)
        
        # Create two Data Link objects.  One is for the spotlight
        # and one is for the dongle.
        self.spotlightDataLink = DataLink(self, DEFAULT_SPOTLIGHT_SERIALPORT)
        self.dongleDataLink = DataLink(self, DEFAULT_DONGLE_SERIALPORT)

        #
        # Create the data directory objects, which store information
        # about Pushpins in a dictionary that is indexed by
        # NeighborhoodID.  These directories are registered as
        # listeners with the dongleDataLink notification center.
        #
        self.localizationDirectory = NodeDirectory(self)
        self.dumpDirectory = NodeDirectory(self)
        self.dongleDataLink.RegisterListener(self.localizationDirectory, ROUTED_REPLY_MESSAGE)
        self.dongleDataLink.RegisterListener(self.dumpDirectory, IDE_DUMP_MESSAGE)
        
        # Other data model instances.
        self.pushpinOS = None

        # Set standard in and standard error and redirect the output of the print
        # command to print to the local stdout.
        self.pd_stdout = self.mainWindow.stdout
        self.pd_stderr = self.mainWindow.stderr
        sys.stdout = self.pd_stdout

        # Finally, load the Notebook tabs.
        self.mainWindow.loadNotebookPanels()

        # Return a success flag
        return true

#---------------------------------------------------------------------------

pdApp = PushpinDebuggerApp(0)     # Create an instance of the application class
pdApp.MainLoop()                  # Tell it to start processing events

#----------------------------------------------------------------------------
