#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#
# Purpose:
#   - This class represents a single Pushpin packet composed of a
#     structured header and arbitrary content.
#
# Assumptions:
#
# History:
#   - 06 Apr 2004 by Josh Lifton.
#

import string
from binascii import a2b_hex
from binascii import b2a_hex
from DataFidelity import crc8
from MessageDecoder import *

PACKET_HEADER_SIZE = 8
PACKET_HEADER_SIZE_WITHOUT_CRCS = 6

class PushpinPacket :

    def __init__(self,
                 packetList=[]) :
        self.packetList = packetList
        self.extractPacketFromHexList()

    def extractPacketFromHexList(self) :
        self.isComplete = False
        self.isCorrect = False
        self.toAddress = None
        self.fromAddress = None
        self.packetType = None
        self.sequenceNumber = None
        self.contentSize = None
        self.headerCRC = None
        self.contentCRC = None
        self.headerList = []
        self.contentList = []
        self.actualHeaderCRC = None
        self.actualContentCRC = None
        for i in range(len(self.packetList)) :
            if i == 0 :
                self.toAddress = self.packetList[i].upper()
            elif i == 1 :
                self.fromAddress = self.packetList[i].upper()
            elif i == 2 :
                self.packetType = self.packetList[i].upper()
            elif i == 3 :
                self.sequenceNumber = self.packetList[i].upper()
            elif i == 4 :
                self.contentSize = self.packetList[i].upper()
            elif i == 5 :
                self.contentSize = self.packetList[i].upper() + self.contentSize
            elif i == 6 :
                self.headerCRC = self.packetList[i].upper()
            elif i == 7 :
                self.contentCRC = self.packetList[i].upper()
                self.headerList = self.packetList[:PACKET_HEADER_SIZE]
                self.contentList = self.packetList[PACKET_HEADER_SIZE:]
                if int(self.contentSize,16) <= len(self.packetList[PACKET_HEADER_SIZE:]) :
                    # Ignore content beyond contentSize.
                    self.contentList = self.contentList[:int(self.contentSize,16)]
                    self.isComplete = True
                    self.actualHeaderCRC = str('%02X' % crc8(a2b_hex(string.join(self.headerList[0:PACKET_HEADER_SIZE_WITHOUT_CRCS], sep='')))).upper()
                    self.actualContentCRC = str('%02X' % crc8(a2b_hex(string.join(self.contentList, sep='')))).upper()
                    if (self.headerCRC == self.actualHeaderCRC) and (self.contentCRC == self.actualContentCRC) :
                        self.isCorrect = True
                        # If the packet has any content, we assume that the first byte of the
                        # content contains the message type.  We use this message type to select
                        # the correct decoder for parsing the packet.
                        if len(self.contentList) > 0:
                            self.decoder = getDecoderForMessageType(self.contentList[0])
                break
    
    def addBytes(self, byteString) :
        # Expects a string of ASCII data.
        if self.isComplete or not byteString :
            return
        for i in range(len(byteString)) :
            self.packetList.append(b2a_hex(byteString[i]).upper())
            if len(self.packetList) >= PACKET_HEADER_SIZE :
                self.extractPacketFromHexList()
                if self.isComplete :
                    # This packet is complete, give the remaining bytes back.
                    return byteString[i+1:]
        return ''
            



    def printHeader(self) :
        if self.headerList :
            print('================================ Start header =============')
            print('      toAddress: ' + self.toAddress)
            print('    fromAddress: ' + self.fromAddress)
            print('     packetType: ' + self.packetType)
            print(' sequenceNumber: ' + self.sequenceNumber)
            print('    contentSize: ' + self.contentSize)
            if self.headerCRC == self.actualHeaderCRC :
                print('      headerCRC: ' + self.headerCRC + ' PASSED')
            else :
                print('      headerCRC: ' + self.headerCRC + ' FAILED.  Computed headerCRC: ' + self.actualHeaderCRC)
            if self.contentCRC == self.actualContentCRC :
                print('      contentCRC: ' + self.contentCRC + ' PASSED')
            else :
                print('      contentCRC: ' + self.contentCRC + ' FAILED.  Computed contentCRC: ' + self.actualContentCRC)
            print('================================ End header ===============')
        else :
            print('*** INCOMPLETE HEADER ***')
            print(self.packetList[:PACKET_HEADER_SIZE])

    def getMessageType(self):
        return self.contentList[0]

    def getMessageLegend(self):
        return self.decoder.getFieldStringsAsList()

    #
    # Parse the packet content into a list by correlating the
    # message type (first byte of the content) with packet decoder definitions
    # in MessageDecoder.py.
    #
    # If the asStrings argument is true, the returned list will contain
    # the ASCII byte values of each field as a string.  Otherwise, the list
    # contains fields that have been decoded and placed into the correct
    # type (int, float, long, etc.)
    #
    # Returns the parsed list.
    #
    def parseIntoList(self, asStrings = True):
        contentPtr = 0
        parsedList = []

        if self.decoder == None:
            print >> controller.pd_stderr, ("Error in parsePacketIntoList(): ", 
                                            "Attempted to parse an unknown packet type")
            return []

        #
        # Decode the packet.
        #
        for field in range( self.decoder.getNumberOfFields() ):

            # Unsigned chars
            if (self.decoder.getFieldType(field) == PD_UCHAR):
                if asStrings:
                    value = self.contentList[contentPtr]
                else:
                    value = int(self.contentList[contentPtr], 16)
                contentPtr += 1
                
            # Unsigned Integers
            elif (self.decoder.getFieldType(field) == PD_UINT):
                if asStrings:
                    value = (self.contentList[contentPtr + 1] +
                             self.contentList[contentPtr])
                else:
                    value = int(self.contentList[contentPtr + 1] +
                                self.contentList[contentPtr], 16)
                contentPtr += 2
                
            # Signed Integers
            elif (self.decoder.getFieldType(field) == PD_SINT):
                if asStrings:
                    value = (self.contentList[contentPtr + 1] +
                             self.contentList[contentPtr])
                else:
                    value = int(self.contentList[contentPtr + 1] +
                                self.contentList[contentPtr], 16)
                contentPtr += 2

            # Unsigned Longs
            elif (self.decoder.getFieldType(field) == PD_ULONG):
                if asStrings:
                    value = (self.contentList[contentPtr + 3] +
                             self.contentList[contentPtr + 2] +
                             self.contentList[contentPtr + 1] +
                             self.contentList[contentPtr] )
                else:
                    value = long(self.contentList[contentPtr + 3] +
                                 self.contentList[contentPtr + 2] +
                                 self.contentList[contentPtr + 1] +
                                 self.contentList[contentPtr], 16)
                contentPtr += 4
                
            # Unsigned Floats
            #
            # Meme - This is going to need to be fixed!!! I'm going to have to write my
            # own bytecode to float converter to python??
            #
            elif (self.decoder.getSizeForField(field) == PD_FLOAT):
                if asStrings:
                    value = (self.contentList[contentPtr] +
                             self.contentList[contentPtr + 1] +
                             self.contentList[contentPtr + 2] +
                             self.contentList[contentPtr + 3] )
                else:
                    value = (self.contentList[contentPtr] +
                             self.contentList[contentPtr + 1] +
                             self.contentList[contentPtr + 2] +
                             self.contentList[contentPtr + 3] )
                contentPtr += 4

            parsedList.append(value)
        return parsedList


    def getDictionaryKey(self):
        return self.contentList[ self.decoder.getDictionaryKeyLocation() ] 
